@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2017                                     CNRS
- * Copyright (C) 2017                                     Inria
- * Copyright (C) 2015                                     Université de Bordeaux
+ * Copyright (C) 2015-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -32,9 +30,10 @@
 
 #define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 
-void matrix_fill(void *buffers[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+void matrix_fill(void *buffers[], void *cl_arg)
 {
 	unsigned i, j;
+	(void)cl_arg;
 
 	/* length of the matrix */
 	unsigned nx = STARPU_MATRIX_GET_NX(buffers[0]);
@@ -99,11 +98,21 @@ struct starpu_codelet cl_check_scale =
 	.name = "fmultiple_check_scale"
 };
 
-void empty(void *buffers[] STARPU_ATTRIBUTE_UNUSED, void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+void empty(void *buffers[], void *cl_arg)
 {
 	/* This doesn't need to do anything, it's simply used to make coherency
 	 * between the two views, by simply running on the home node of the
 	 * data, thus getting back all data pieces there.  */
+	(void)buffers;
+	(void)cl_arg;
+
+	/* This check is just for testsuite */
+	int node = starpu_task_get_current_data_node(0);
+	unsigned i;
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(starpu_task_get_current());
+	STARPU_ASSERT(node >= 0);
+	for (i = 1; i < nbuffers; i++)
+		STARPU_ASSERT(starpu_task_get_current_data_node(i) == node);
 }
 
 struct starpu_codelet cl_switch =
@@ -113,7 +122,7 @@ struct starpu_codelet cl_switch =
 	.name = "switch"
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
 	unsigned j, n=1;
 	int matrix[NX][NY];
