@@ -1,9 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2008-2015,2017-2018                      Université de Bordeaux
- * Copyright (C) 2010                                     Mehdi Juhoor
- * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -608,8 +607,8 @@ void dw_callback_codelet_update_u12_21(void *argcb)
 		unsigned nblocks = args->nblocks;
 
 		/* the number of tasks to be done */
-		unsigned *remaining = malloc(sizeof(unsigned));
-		*remaining = (nblocks - 1 - i)*(nblocks - 1 - i);
+		unsigned *remaining_tasks = malloc(sizeof(unsigned));
+		*remaining_tasks = (nblocks - 1 - i)*(nblocks - 1 - i);
 
 		unsigned slicey, slicex;
 		for (slicey = i+1; slicey < nblocks; slicey++)
@@ -633,7 +632,7 @@ void dw_callback_codelet_update_u12_21(void *argcb)
 				u22a->j = slicey;
 				u22a->dataA = args->dataA;
 				u22a->nblocks = nblocks;
-				u22a->remaining = remaining;
+				u22a->remaining = remaining_tasks;
 
 				task22->handles[0] = starpu_data_get_sub_data(args->dataA, 2, u22a->i, u22a->k);
 				task22->handles[1] = starpu_data_get_sub_data(args->dataA, 2, u22a->k, u22a->j);
@@ -645,7 +644,7 @@ void dw_callback_codelet_update_u12_21(void *argcb)
 			}
 		}
 
-		free(remaining);
+		free(remaining_tasks);
 	}
 }
 
@@ -798,7 +797,7 @@ void initialize_system(float **A, float **B, unsigned dim, unsigned pinned)
 	}
 }
 
-void free_system(float *A, float *B, unsigned dim, unsigned pinned)
+void free_system(float *A, float *B, unsigned pinned)
 {
 	if (pinned)
 	{
