@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012,2014,2016                      Inria
- * Copyright (C) 2009-2017                                Université de Bordeaux
- * Copyright (C) 2010-2017                                CNRS
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,12 +19,13 @@
 
 #include <datawizard/datawizard.h>
 #include <datawizard/memory_nodes.h>
+#include <datawizard/memstats.h>
+#include <datawizard/malloc.h>
 #include <core/dependencies/data_concurrency.h>
 #include <common/uthash.h>
 #include <common/starpu_spinlock.h>
 #include <core/task.h>
 #include <core/workers.h>
-#include <datawizard/memstats.h>
 #ifdef STARPU_OPENMP
 #include <util/openmp_runtime_support.h>
 #endif
@@ -59,7 +58,7 @@ void _starpu_data_interface_init(void)
 
 void _starpu_data_interface_shutdown()
 {
-	struct handle_entry *entry, *tmp;
+	struct handle_entry *entry=NULL, *tmp=NULL;
 
 	if (registered_handles)
 	{
@@ -86,7 +85,7 @@ void _starpu_data_interface_shutdown()
 void _starpu_omp_unregister_region_handles(struct starpu_omp_region *region)
 {
 	_starpu_spin_lock(&region->registered_handles_lock);
-	struct handle_entry *entry, *tmp;
+	struct handle_entry *entry=NULL, *tmp=NULL;
 	HASH_ITER(hh, (region->registered_handles), entry, tmp)
 	{
 		entry->handle->removed_from_context_hash = 1;
@@ -99,7 +98,7 @@ void _starpu_omp_unregister_region_handles(struct starpu_omp_region *region)
 
 void _starpu_omp_unregister_task_handles(struct starpu_omp_task *task)
 {
-	struct handle_entry *entry, *tmp;
+	struct handle_entry *entry=NULL, *tmp=NULL;
 	HASH_ITER(hh, task->registered_handles, entry, tmp)
 	{
 		entry->handle->removed_from_context_hash = 1;
@@ -159,7 +158,7 @@ void _starpu_data_register_ram_pointer(starpu_data_handle_t handle, void *ptr)
 	struct starpu_omp_task *task = _starpu_omp_get_task();
 	if (task)
 	{
-		if (task->is_implicit)
+		if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
 		{
 			struct starpu_omp_region *parallel_region = task->owner_region;
 			_starpu_spin_lock(&parallel_region->registered_handles_lock);
@@ -204,7 +203,7 @@ starpu_data_handle_t starpu_data_lookup(const void *ptr)
 	struct starpu_omp_task *task = _starpu_omp_get_task();
 	if (task)
 	{
-		if (task->is_implicit)
+		if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
 		{
 			struct starpu_omp_region *parallel_region = task->owner_region;
 			_starpu_spin_lock(&parallel_region->registered_handles_lock);
@@ -261,7 +260,7 @@ static void _starpu_register_new_data(starpu_data_handle_t handle,
 	STARPU_ASSERT(handle);
 
 	/* initialize the new lock */
-	_starpu_data_requester_list_init(&handle->req_list);
+	_starpu_data_requester_prio_list_init(&handle->req_list);
 	handle->refcnt = 0;
 	handle->unlocking_reqs = 0;
 	handle->busy_count = 0;
@@ -279,8 +278,15 @@ static void _starpu_register_new_data(starpu_data_handle_t handle,
 	handle->switch_cl = NULL;
 	handle->partitioned = 0;
 	handle->readonly = 0;
+	handle->active = 1;
+	handle->active_ro = 0;
 	handle->root_handle = handle;
 	handle->father_handle = NULL;
+	handle->active_children = NULL;
+	handle->active_readonly_children = NULL;
+	handle->nactive_readonly_children = 0;
+	handle->nsiblings = 0;
+	handle->siblings = NULL;
 	handle->sibling_index = 0; /* could be anything for the root */
 	handle->depth = 1; /* the tree is just a node yet */
         handle->mpi_data = NULL; /* invalid until set */
@@ -290,6 +296,7 @@ static void _starpu_register_new_data(starpu_data_handle_t handle,
 	handle->sequential_consistency =
 		starpu_data_get_default_sequential_consistency_flag();
 	handle->initialized = home_node != -1;
+	handle->ooc = 1;
 
 	STARPU_PTHREAD_MUTEX_INIT(&handle->sequential_consistency_mutex, NULL);
 	handle->last_submitted_mode = STARPU_R;
@@ -308,7 +315,7 @@ static void _starpu_register_new_data(starpu_data_handle_t handle,
 	handle->init_cl = NULL;
 
 	handle->reduction_refcnt = 0;
-	_starpu_data_requester_list_init(&handle->reduction_req_list);
+	_starpu_data_requester_prio_list_init(&handle->reduction_req_list);
 	handle->reduction_tmp_handles = NULL;
 	handle->write_invalidation_req = NULL;
 
@@ -331,7 +338,7 @@ static void _starpu_register_new_data(starpu_data_handle_t handle,
 		starpu_data_assign_arbiter(handle, _starpu_global_arbiter);
 	else
 		handle->arbiter = NULL;
-	_starpu_data_requester_list_init(&handle->arbitered_req_list);
+	_starpu_data_requester_prio_list_init(&handle->arbitered_req_list);
 	handle->last_locality = -1;
 
 	/* that new data is invalid from all nodes perpective except for the
@@ -369,12 +376,14 @@ static void _starpu_register_new_data(starpu_data_handle_t handle,
 	/* now the data is available ! */
 	_starpu_spin_unlock(&handle->header_lock);
 
-
-
-	ptr = starpu_data_handle_to_pointer(handle, STARPU_MAIN_RAM);
-	if (ptr != NULL)
+	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
-		_starpu_data_register_ram_pointer(handle, ptr);
+		if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
+			continue;
+
+		ptr = starpu_data_handle_to_pointer(handle, node);
+		if (ptr != NULL)
+			_starpu_data_register_ram_pointer(handle, ptr);
 	}
 }
 
@@ -400,7 +409,7 @@ _starpu_data_initialize_per_worker(starpu_data_handle_t handle)
 		replicate->state = STARPU_INVALID;
 		replicate->refcnt = 0;
 		replicate->handle = handle;
-		replicate->requested = 0;
+		//replicate->nb_tasks_prefetch = 0;
 
 		for (node = 0; node < STARPU_MAXNODES; node++)
 		{
@@ -441,6 +450,7 @@ int _starpu_data_handle_init(starpu_data_handle_t handle, struct starpu_data_int
 	handle->ops = interface_ops;
 	handle->mf_node = mf_node;
 	handle->mpi_data = NULL;
+	handle->partition_automatic_disabled = 0;
 
 	size_t interfacesize = interface_ops->interface_size;
 
@@ -456,6 +466,7 @@ int _starpu_data_handle_init(starpu_data_handle_t handle, struct starpu_data_int
 		replicate->handle = handle;
 
 		_STARPU_CALLOC(replicate->data_interface, 1, interfacesize);
+		if (handle->ops->init) handle->ops->init(replicate->data_interface);
 	}
 
 	return 0;
@@ -479,6 +490,9 @@ void starpu_data_register(starpu_data_handle_t *handleptr, int home_node,
 	STARPU_ASSERT(handleptr);
 	*handleptr = handle;
 
+	/* check the interfaceid is set */
+	STARPU_ASSERT(ops->interfaceid != STARPU_UNKNOWN_INTERFACE_ID);
+
 	/* fill the interface fields with the appropriate method */
 	STARPU_ASSERT(ops->register_data_handle);
 	ops->register_data_handle(handle, home_node, data_interface);
@@ -497,8 +511,15 @@ void *starpu_data_handle_to_pointer(starpu_data_handle_t handle, unsigned node)
 {
 	/* Check whether the operation is supported and the node has actually
 	 * been allocated.  */
-	if (handle->ops->handle_to_pointer
-	    && starpu_data_test_if_allocated_on_node(handle, node))
+	if (!starpu_data_test_if_allocated_on_node(handle, node))
+		return NULL;
+	if (handle->ops->to_pointer)
+	{
+		return handle->ops->to_pointer(starpu_data_get_interface_on_node(handle, node), node);
+	}
+
+	/* Deprecated */
+	if (handle->ops->handle_to_pointer)
 	{
 		return handle->ops->handle_to_pointer(handle, node);
 	}
@@ -522,8 +543,7 @@ int starpu_data_pointer_is_inside(starpu_data_handle_t handle, unsigned node, vo
 
 void *starpu_data_get_local_ptr(starpu_data_handle_t handle)
 {
-	return starpu_data_handle_to_pointer(handle,
-					_starpu_memory_node_get_local_key());
+	return starpu_data_handle_to_pointer(handle, starpu_worker_get_local_memory_node());
 }
 
 struct starpu_data_interface_ops* starpu_data_get_interface_ops(starpu_data_handle_t handle)
@@ -534,14 +554,17 @@ struct starpu_data_interface_ops* starpu_data_get_interface_ops(starpu_data_hand
 /*
  * Stop monitoring a piece of data
  */
-
-void _starpu_data_unregister_ram_pointer(starpu_data_handle_t handle)
+void _starpu_data_unregister_ram_pointer(starpu_data_handle_t handle, unsigned node)
 {
-	const void *ram_ptr = starpu_data_handle_to_pointer(handle, STARPU_MAIN_RAM);
+	if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
+		return;
+
 #ifdef STARPU_OPENMP
 	if (handle->removed_from_context_hash)
 		return;
 #endif
+	const void *ram_ptr = starpu_data_handle_to_pointer(handle, node);
+
 	if (ram_ptr != NULL)
 	{
 		/* Remove the PTR -> HANDLE mapping.  If a mapping from PTR
@@ -552,7 +575,7 @@ void _starpu_data_unregister_ram_pointer(starpu_data_handle_t handle)
 		struct starpu_omp_task *task = _starpu_omp_get_task();
 		if (task)
 		{
-			if (task->is_implicit)
+			if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
 			{
 				struct starpu_omp_region *parallel_region = task->owner_region;
 				_starpu_spin_lock(&parallel_region->registered_handles_lock);
@@ -596,6 +619,9 @@ void _starpu_data_free_interfaces(starpu_data_handle_t handle)
 	unsigned node;
 	unsigned nworkers = starpu_worker_get_count();
 
+	if (handle->ops->unregister_data_handle)
+		handle->ops->unregister_data_handle(handle);
+
 	for (node = 0; node < STARPU_MAXNODES; node++)
 		free(handle->per_node[node].data_interface);
 
@@ -655,7 +681,7 @@ int __starpu_data_check_not_busy(starpu_data_handle_t handle)
 }
 
 static
-void _starpu_check_if_valid_and_fetch_data_on_node(starpu_data_handle_t handle, struct _starpu_data_replicate *replicate)
+void _starpu_check_if_valid_and_fetch_data_on_node(starpu_data_handle_t handle, struct _starpu_data_replicate *replicate, const char *origin)
 {
 	unsigned node;
 	unsigned nnodes = starpu_memory_nodes_get_count();
@@ -671,9 +697,9 @@ void _starpu_check_if_valid_and_fetch_data_on_node(starpu_data_handle_t handle,
 	}
 	if (valid)
 	{
-		int ret = _starpu_fetch_data_on_node(handle, handle->home_node, replicate, STARPU_R, 0, 0, 0, NULL, NULL);
+		int ret = _starpu_fetch_data_on_node(handle, handle->home_node, replicate, STARPU_R, 0, STARPU_FETCH, 0, NULL, NULL, 0, origin);
 		STARPU_ASSERT(!ret);
-		_starpu_release_data_on_node(handle, handle->home_node, replicate);
+		_starpu_release_data_on_node(handle, 0, replicate);
 	}
 	else
 	{
@@ -693,7 +719,7 @@ static void _starpu_data_unregister_fetch_data_callback(void *_arg)
 
 	struct _starpu_data_replicate *replicate = &handle->per_node[arg->memory_node];
 
-	_starpu_check_if_valid_and_fetch_data_on_node(handle, replicate);
+	_starpu_check_if_valid_and_fetch_data_on_node(handle, replicate, "_starpu_data_unregister_fetch_data_callback");
 
 	/* unlock the caller */
 	STARPU_PTHREAD_MUTEX_LOCK(&arg->mutex);
@@ -702,6 +728,12 @@ static void _starpu_data_unregister_fetch_data_callback(void *_arg)
 	STARPU_PTHREAD_MUTEX_UNLOCK(&arg->mutex);
 }
 
+void _starpu_data_set_unregister_hook(starpu_data_handle_t handle, _starpu_data_handle_unregister_hook func)
+{
+ 	STARPU_ASSERT(handle->unregister_hook == NULL);
+	handle->unregister_hook = func;
+}
+
 /* Unregister the data handle, perhaps we don't need to update the home_node
  * (in that case coherent is set to 0)
  * nowait is for internal use when we already know for sure that we won't have to wait.
@@ -718,6 +750,10 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 	int sequential_consistency = handle->sequential_consistency;
 	if (sequential_consistency && !nowait)
 	{
+		/* We will acquire it in write mode to catch all dependencies,
+		 * but possibly it's not actually initialized. Fake it to avoid
+		 getting caught doing it */
+		handle->initialized = 1;
 		STARPU_ASSERT_MSG(_starpu_worker_may_perform_blocking_calls(), "starpu_data_unregister must not be called from a task or callback, perhaps you can use starpu_data_unregister_submit instead");
 
 		/* If sequential consistency is enabled, wait until data is available */
@@ -745,7 +781,7 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 			{
 				/* no one has locked this data yet, so we proceed immediately */
 				struct _starpu_data_replicate *home_replicate = &handle->per_node[home_node];
-				_starpu_check_if_valid_and_fetch_data_on_node(handle, home_replicate);
+				_starpu_check_if_valid_and_fetch_data_on_node(handle, home_replicate, "_starpu_data_unregister");
 			}
 			else
 			{
@@ -762,16 +798,15 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 		 * this piece of data back into the CPU format.
 		 * XXX : This is quite hacky, could we submit a task instead ?
 		 */
-		if (_starpu_data_is_multiformat_handle(handle) &&
-			(  starpu_node_get_kind(handle->mf_node) != STARPU_CPU_RAM
-			&& starpu_node_get_kind(handle->mf_node) != STARPU_SCC_RAM
-			&& starpu_node_get_kind(handle->mf_node) != STARPU_SCC_SHM
-			 ))
+		if (_starpu_data_is_multiformat_handle(handle) && (starpu_node_get_kind(handle->mf_node) != STARPU_CPU_RAM))
 		{
 			_STARPU_DEBUG("Conversion needed\n");
 			void *buffers[1];
 			struct starpu_multiformat_interface *format_interface;
-			format_interface = (struct starpu_multiformat_interface *) starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+			home_node = handle->home_node;
+			if (home_node < 0 || (starpu_node_get_kind(home_node) != STARPU_CPU_RAM))
+				home_node = STARPU_MAIN_RAM;
+			format_interface = (struct starpu_multiformat_interface *) starpu_data_get_interface_on_node(handle, home_node);
 			struct starpu_codelet *cl = NULL;
 			enum starpu_node_kind node_kind = starpu_node_get_kind(handle->mf_node);
 
@@ -805,8 +840,6 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 				}
 #endif
 				case STARPU_CPU_RAM:      /* Impossible ! */
-				case STARPU_SCC_RAM:      /* Impossible ! */
-				case STARPU_SCC_SHM:      /* Impossible ! */
 				default:
 					STARPU_ABORT();
 			}
@@ -853,6 +886,13 @@ retry_busy:
 	}
 	STARPU_PTHREAD_MUTEX_UNLOCK(&handle->busy_mutex);
 
+	/* Unregister MPI things after having waiting for MPI reqs etc. to settle down */
+	if (handle->unregister_hook)
+	{
+		handle->unregister_hook(handle);
+		handle->unregister_hook = NULL;
+	}
+
 	/* Wait for finished requests to release the handle */
 	_starpu_spin_lock(&handle->header_lock);
 	if (handle->busy_count)
@@ -862,18 +902,22 @@ retry_busy:
 		goto retry_busy;
 	}
 
-	size_t size = _starpu_data_get_size(handle);
-
-	_starpu_data_unregister_ram_pointer(handle);
+	size_t size = _starpu_data_get_alloc_size(handle);
 
 	/* Destroy the data now */
 	unsigned node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct _starpu_data_replicate *local = &handle->per_node[node];
+		STARPU_ASSERT(!local->refcnt);
+		if (local->allocated)
+		{
+			_starpu_data_unregister_ram_pointer(handle, node);
+
 		/* free the data copy in a lazy fashion */
-		if (local->allocated && local->automatically_allocated)
-			_starpu_request_mem_chunk_removal(handle, local, node, size);
+			if (local->automatically_allocated)
+				_starpu_request_mem_chunk_removal(handle, local, node, size);
+		}
 	}
 	if (handle->per_worker)
 	{
@@ -882,6 +926,7 @@ retry_busy:
 		for (worker = 0; worker < nworkers; worker++)
 		{
 			struct _starpu_data_replicate *local = &handle->per_worker[worker];
+			STARPU_ASSERT(!local->refcnt);
 			/* free the data copy in a lazy fashion */
 			if (local->allocated && local->automatically_allocated)
 				_starpu_request_mem_chunk_removal(handle, local, starpu_worker_get_memory_node(worker), size);
@@ -895,6 +940,7 @@ retry_busy:
 	_starpu_spin_destroy(&handle->header_lock);
 
 	_starpu_data_clear_implicit(handle);
+	free(handle->active_readonly_children);
 
 	STARPU_PTHREAD_MUTEX_DESTROY(&handle->busy_mutex);
 	STARPU_PTHREAD_COND_DESTROY(&handle->busy_cond);
@@ -917,21 +963,12 @@ void starpu_data_unregister(starpu_data_handle_t handle)
 	STARPU_ASSERT_MSG(handle->magic == 42, "data %p is invalid (was it already registered?)", handle);
 	STARPU_ASSERT_MSG(!handle->lazy_unregister, "data %p can not be unregistered twice", handle);
 
-	if (handle->unregister_hook)
-	{
-		handle->unregister_hook(handle);
-	}
-
 	_starpu_data_unregister(handle, 1, 0);
 }
 
 void starpu_data_unregister_no_coherency(starpu_data_handle_t handle)
 {
 	STARPU_ASSERT_MSG(handle->magic == 42, "data %p is invalid (was it already registered?)", handle);
-	if (handle->unregister_hook)
-	{
-		handle->unregister_hook(handle);
-	}
 
 	_starpu_data_unregister(handle, 0, 0);
 }
@@ -948,7 +985,7 @@ static void _starpu_data_unregister_submit_cb(void *arg)
 	STARPU_ASSERT(handle->busy_count);
         _starpu_spin_unlock(&handle->header_lock);
 
-	starpu_data_release_on_node(handle, STARPU_ACQUIRE_ALL_NODES);
+	starpu_data_release_on_node(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL);
 }
 
 void starpu_data_unregister_submit(starpu_data_handle_t handle)
@@ -956,19 +993,14 @@ void starpu_data_unregister_submit(starpu_data_handle_t handle)
 	STARPU_ASSERT_MSG(handle->magic == 42, "data %p is invalid (was it already registered?)", handle);
 	STARPU_ASSERT_MSG(!handle->lazy_unregister, "data %p can not be unregistered twice", handle);
 
-	if (handle->unregister_hook)
-	{
-		handle->unregister_hook(handle);
-	}
-
 	/* Wait for all task dependencies on this handle before putting it for free */
-	starpu_data_acquire_on_node_cb(handle, STARPU_ACQUIRE_ALL_NODES, handle->initialized?STARPU_RW:STARPU_W, _starpu_data_unregister_submit_cb, handle);
+	starpu_data_acquire_on_node_cb(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL, handle->initialized?STARPU_RW:STARPU_W, _starpu_data_unregister_submit_cb, handle);
 }
 
 static void _starpu_data_invalidate(void *data)
 {
 	starpu_data_handle_t handle = data;
-	size_t size = _starpu_data_get_size(handle);
+	size_t size = _starpu_data_get_alloc_size(handle);
 	_starpu_spin_lock(&handle->header_lock);
 
 	//_STARPU_DEBUG("Really invalidating data %p\n", data);
@@ -990,15 +1022,14 @@ static void _starpu_data_invalidate(void *data)
 
 		if (local->mc && local->allocated && local->automatically_allocated)
 		{
-			if (node == STARPU_MAIN_RAM)
-				_starpu_data_unregister_ram_pointer(handle);
+			_starpu_data_unregister_ram_pointer(handle, node);
 
 			/* free the data copy in a lazy fashion */
 			_starpu_request_mem_chunk_removal(handle, local, node, size);
 		}
 
 		if (local->state != STARPU_INVALID)
-			_STARPU_TRACE_DATA_INVALIDATE(handle, node);
+			_STARPU_TRACE_DATA_STATE_INVALID(handle, node);
 		local->state = STARPU_INVALID;
 	}
 
@@ -1020,14 +1051,14 @@ static void _starpu_data_invalidate(void *data)
 
 	_starpu_spin_unlock(&handle->header_lock);
 
-	starpu_data_release_on_node(handle, STARPU_ACQUIRE_ALL_NODES);
+	starpu_data_release_on_node(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL);
 }
 
 void starpu_data_invalidate(starpu_data_handle_t handle)
 {
 	STARPU_ASSERT(handle);
 
-	starpu_data_acquire_on_node(handle, STARPU_ACQUIRE_ALL_NODES, STARPU_W);
+	starpu_data_acquire_on_node(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL, STARPU_W);
 
 	_starpu_data_invalidate(handle);
 
@@ -1038,7 +1069,7 @@ void starpu_data_invalidate_submit(starpu_data_handle_t handle)
 {
 	STARPU_ASSERT(handle);
 
-	starpu_data_acquire_on_node_cb(handle, STARPU_ACQUIRE_ALL_NODES, STARPU_W, _starpu_data_invalidate, handle);
+	starpu_data_acquire_on_node_cb(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL, STARPU_W, _starpu_data_invalidate, handle);
 
 	handle->initialized = 0;
 }
@@ -1062,15 +1093,14 @@ int starpu_data_interface_get_next_id(void)
 int starpu_data_pack(starpu_data_handle_t handle, void **ptr, starpu_ssize_t *count)
 {
 	STARPU_ASSERT_MSG(handle->ops->pack_data, "The datatype interface %s (%d) does not have a pack operation", handle->ops->name, handle->ops->interfaceid);
-	return handle->ops->pack_data(handle, _starpu_memory_node_get_local_key(), ptr, count);
+	return handle->ops->pack_data(handle, starpu_worker_get_local_memory_node(), ptr, count);
 }
 
 int starpu_data_unpack(starpu_data_handle_t handle, void *ptr, size_t count)
 {
 	STARPU_ASSERT_MSG(handle->ops->unpack_data, "The datatype interface %s (%d) does not have an unpack operation", handle->ops->name, handle->ops->interfaceid);
 	int ret;
-	ret = handle->ops->unpack_data(handle, _starpu_memory_node_get_local_key(), ptr, count);
-	starpu_free_flags(ptr, count, 0);
+	ret = handle->ops->unpack_data(handle, starpu_worker_get_local_memory_node(), ptr, count);
 	return ret;
 }
 
@@ -1079,13 +1109,36 @@ size_t starpu_data_get_size(starpu_data_handle_t handle)
 	return handle->ops->get_size(handle);
 }
 
-void starpu_data_set_name(starpu_data_handle_t handle, const char *name)
+size_t starpu_data_get_alloc_size(starpu_data_handle_t handle)
+{
+	if (handle->ops->get_alloc_size)
+		return handle->ops->get_alloc_size(handle);
+	else
+		return handle->ops->get_size(handle);
+}
+
+void starpu_data_set_name(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED, const char *name STARPU_ATTRIBUTE_UNUSED)
 {
 	_STARPU_TRACE_DATA_NAME(handle, name);
 }
 
-void starpu_data_set_coordinates_array(starpu_data_handle_t handle, int dimensions, int dims[])
+int starpu_data_get_home_node(starpu_data_handle_t handle)
 {
+	return handle->home_node;
+}
+
+void starpu_data_set_coordinates_array(starpu_data_handle_t handle, unsigned dimensions, int dims[])
+{
+	unsigned i;
+	unsigned max_dimensions = sizeof(handle->coordinates)/sizeof(handle->coordinates[0]);
+
+	if (dimensions > max_dimensions)
+		dimensions = max_dimensions;
+
+	handle->dimensions = dimensions;
+	for (i = 0; i < dimensions; i++)
+		handle->coordinates[i] = dims[i];
+
 	_STARPU_TRACE_DATA_COORDINATES(handle, dimensions, dims);
 }
 
@@ -1102,3 +1155,16 @@ void starpu_data_set_coordinates(starpu_data_handle_t handle, unsigned dimension
 
 	starpu_data_set_coordinates_array(handle, dimensions, dims);
 }
+
+unsigned starpu_data_get_coordinates_array(starpu_data_handle_t handle, unsigned dimensions, int dims[])
+{
+	unsigned i;
+
+	if (dimensions > handle->dimensions)
+		dimensions = handle->dimensions;
+
+	for (i = 0; i < dimensions; i++)
+		dims[i] = handle->coordinates[i];
+
+	return dimensions;
+}
