@@ -1,8 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2016                           Inria
- * Copyright (C) 2012,2014-2015                           CNRS
- * Copyright (C) 2010-2017                                Université de Bordeaux
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2017-2019  Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,12 +23,56 @@
 
 #ifdef STARPU_USE_FXT
 
-void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
+#ifdef STARPU_HAVE_POTI
+#ifdef HAVE_POTI_INIT_CUSTOM
+int _starpu_poti_extendedSetState = -1;
+int _starpu_poti_semiExtendedSetState = -1;
+int _starpu_poti_MemoryEvent = -1;
+int _starpu_poti_MpiLinkStart = -1;
+#endif
+#endif
+
+void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct starpu_fxt_options *options)
 {
 	unsigned i;
 #ifdef STARPU_HAVE_POTI
 #ifdef HAVE_POTI_INIT_CUSTOM
 	poti_header();     /* see poti_init_custom to customize the header */
+	_starpu_poti_extendedSetState = poti_header_DeclareEvent (PAJE_SetState,
+						     11,
+						     "Size string",
+						     "Params string",
+						     "Footprint string",
+						     "Tag string",
+						     "JobId string",
+						     "SubmitOrder string",
+						     "GFlop string",
+						     "X string",
+						     "Y string",
+						     /* "Z string", */
+						     "Iteration string",
+						     "Subiteration string");
+	_starpu_poti_semiExtendedSetState = poti_header_DeclareEvent (PAJE_SetState,
+						     6,
+						     "Size string",
+						     "Params string",
+						     "Footprint string",
+						     "Tag string",
+						     "JobId string",
+						     "SubmitOrder string"
+						     );
+#ifdef HAVE_POTI_USER_NEWEVENT
+	if (options->memory_states)
+	{
+		_starpu_poti_MemoryEvent = poti_header_DeclareEvent (PAJE_NewEvent,
+							     4,
+							     "Handle string",
+							     "Info string",
+							     "Size string",
+							     "Dest string");
+	}
+	_starpu_poti_MpiLinkStart = poti_header_DeclareEvent(PAJE_StartLink, 1, "MPITAG string");
+#endif
 #else
 	poti_header(1,1);
 #endif
@@ -141,20 +184,61 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 	fprintf(file, "%%	Container	string\n");
 	fprintf(file, "%%	Type	string\n");
 	fprintf(file, "%%	Value	string\n");
-#if 0
-	/* TODO: implement in worker_set_detailed_state() and handle_codelet_details() */
 	fprintf(file, "%%	Size	string\n");
 	fprintf(file, "%%	Params	string\n");
 	fprintf(file, "%%	Footprint	string\n");
 	fprintf(file, "%%	Tag	string\n");
 	fprintf(file, "%%	JobId	string\n");
+	fprintf(file, "%%	SubmitOrder	string\n");
 	fprintf(file, "%%	GFlop	string\n");
 	fprintf(file, "%%	X	string\n");
 	fprintf(file, "%%	Y	string\n");
-	fprintf(file, "%%	Z	string\n");
+	/* fprintf(file, "%%	Z	string\n"); */
 	fprintf(file, "%%	Iteration	string\n");
 	fprintf(file, "%%	Subiteration	string\n");
-#endif
+	fprintf(file, "%%EndEventDef\n");
+	fprintf(file, "%%EventDef PajeSetState 21\n");
+	fprintf(file, "%%	Time	date\n");
+	fprintf(file, "%%	Container	string\n");
+	fprintf(file, "%%	Type	string\n");
+	fprintf(file, "%%	Value	string\n");
+	fprintf(file, "%%	Size	string\n");
+	fprintf(file, "%%	Params	string\n");
+	fprintf(file, "%%	Footprint	string\n");
+	fprintf(file, "%%	Tag	string\n");
+	fprintf(file, "%%	JobId	string\n");
+	fprintf(file, "%%	SubmitOrder	string\n");
+	fprintf(file, "%%EndEventDef\n");
+	if (options->memory_states)
+	{
+		fprintf(file, "%%EventDef	PajeNewEvent	22\n");
+		fprintf(file, "%%	Time	date\n");
+		fprintf(file, "%%	Type	string\n");
+		fprintf(file, "%%	Container	string\n");
+		fprintf(file, "%%	Value	string\n");
+		fprintf(file, "%%	Handle	string\n");
+		fprintf(file, "%%	Info	string\n");
+		fprintf(file, "%%	Size	string\n");
+		fprintf(file, "%%	Tid	string\n");
+		fprintf(file, "%%EndEventDef\n");
+	}
+	fprintf(file, "%%EventDef	PajeStartLink	23\n");
+	fprintf(file, "%%	Time	date\n");
+	fprintf(file, "%%	Type	string\n");
+	fprintf(file, "%%	Container	string\n");
+	fprintf(file, "%%	Value	string\n");
+	fprintf(file, "%%	StartContainer	string\n");
+	fprintf(file, "%%	Key	string\n");
+	fprintf(file, "%%	MPITAG	string\n");
+	fprintf(file, "%%EndEventDef\n");
+	fprintf(file, "%%EventDef	PajeEndLink	24\n");
+	fprintf(file, "%%	Time	date\n");
+	fprintf(file, "%%	Type	string\n");
+	fprintf(file, "%%	Container	string\n");
+	fprintf(file, "%%	Value	string\n");
+	fprintf(file, "%%	EndContainer	string\n");
+	fprintf(file, "%%	Key	string\n");
+	fprintf(file, "%%	MPITAG	string\n");
 	fprintf(file, "%%EndEventDef\n");
 #endif
 
@@ -169,14 +253,32 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 	poti_DefineContainerType("MPICt", "P", "MPI Communication Thread");
 	poti_DefineContainerType("Sc", "P", "Scheduler");
 	poti_DefineEventType("prog_event", "P", "program event type");
+       poti_DefineEventType("pu", "P", "task push");
+       poti_DefineEventType("po", "P", "task pop");
 	poti_DefineEventType("register", "P", "data registration");
 	poti_DefineEventType("unregister", "P", "data unregistration");
 
 	/* Types for the memory node */
-	poti_DefineEventType("invalidate", "Mm", "data invalidation");
+	poti_DefineEventType("SI", "Mm", "data state invalid");
+	poti_DefineEventType("SS", "Mm", "data state shared");
+	poti_DefineEventType("SO", "Mm", "data state owner");
+	poti_DefineEventType("WU", "Mm", "data wont use");
+	poti_DefineEventType("Al", "Mm", "Allocating Start");
+       poti_DefineEventType("rc", "Mm", "Request Created");
+	poti_DefineEventType("AlE", "Mm", "Allocating End");
+	poti_DefineEventType("Alr", "Mm", "Allocating Async Start");
+	poti_DefineEventType("AlrE", "Mm", "Allocating Async End");
+	poti_DefineEventType("Fe", "Mm", "Free Start");
+	poti_DefineEventType("FeE", "Mm", "Free End");
+	poti_DefineEventType("Wb", "Mm", "WritingBack Start");
+	poti_DefineEventType("WbE", "Mm", "WritingBack End");
+	poti_DefineEventType("DCo", "Mm", "DriverCopy Start");
+	poti_DefineEventType("DCoE", "Mm", "DriverCopy End");
+	poti_DefineEventType("DCoA", "Mm", "DriverCopyAsync Start");
+	poti_DefineEventType("DCoAE", "Mm", "DriverCopyAsync End");
 	poti_DefineVariableType("use", "Mm", "Used (MB)", "0 0 0");
-	poti_DefineVariableType("bwi", "Mm", "Bandwidth In (MB/s)", "0 0 0");
-	poti_DefineVariableType("bwo", "Mm", "Bandwidth Out (MB/s)", "0 0 0");
+	poti_DefineVariableType("bwi_mm", "Mm", "Bandwidth In (MB/s)", "0 0 0");
+	poti_DefineVariableType("bwo_mm", "Mm", "Bandwidth Out (MB/s)", "0 0 0");
 	poti_DefineStateType("MS", "Mm", "Memory Node State");
 	poti_DefineEntityValue("A", "MS", "Allocating", ".4 .1 .0");
 	poti_DefineEntityValue("Ar", "MS", "AllocatingReuse", ".1 .1 .8");
@@ -189,7 +291,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 	poti_DefineEntityValue("No", "MS", "Nothing", ".0 .0 .0");
 
 	/* Types for the Worker of the Memory Node */
-	poti_DefineEventType("user_event", "T", "user event type");
+	poti_DefineEventType("user_event", "P", "user event type");
 	poti_DefineEventType("thread_event", "T", "thread event type");
 	poti_DefineVariableType("gf", "W", "GFlops", "0 0 0");
 	poti_DefineStateType("S", "T", "Thread State");
@@ -207,7 +309,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 	poti_DefineEntityValue("U", "S", "Unpartitioning", ".0 .0 1.0");
 	poti_DefineEntityValue("H", "S", "Hypervisor", ".5 .18 .0");
 	poti_DefineEntityValue("Bu", "S", "Building task", ".5 .18 .0");
-	poti_DefineEntityValue("Su", "S", "Submiting task", ".3 .09 .0");
+	poti_DefineEntityValue("Su", "S", "Submitting task", ".3 .09 .0");
 	poti_DefineEntityValue("Th", "S", "Throttling task submission", ".8 .6 .6");
 	poti_DefineEntityValue("MD", "S", "Decoding task for MPI", ".5 .18 .2");
 	poti_DefineEntityValue("MPr", "S", "Preparing task for MPI", ".4 .14 .2");
@@ -227,24 +329,28 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 	poti_DefineEntityValue("U", "WS", "Unpartitioning", ".0 .0 1.0");
 	poti_DefineEntityValue("H", "WS", "Hypervisor", ".5 .18 .0");
 	poti_DefineEntityValue("Bu", "WS", "Building task", ".5 .18 .0");
-	poti_DefineEntityValue("Su", "WS", "Submiting task", ".3 .09 .0");
+	poti_DefineEntityValue("Su", "WS", "Submitting task", ".3 .09 .0");
 	poti_DefineEntityValue("Th", "WS", "Throttling task submission", ".8 .6 .6");
 
 	/* Types for the MPI Communication Thread of the Memory Node */
 	poti_DefineEventType("MPIev", "MPICt", "MPI event type");
+	poti_DefineVariableType("bwi_mpi", "MPICt", "Bandwidth In (MB/s)", "0 0 0");
+	poti_DefineVariableType("bwo_mpi", "MPICt", "Bandwidth Out (MB/s)", "0 0 0");
 	poti_DefineStateType("CtS", "MPICt", "Communication Thread State");
 	poti_DefineEntityValue("P", "CtS", "Processing", "0 0 0");
+	poti_DefineEntityValue("Pl", "CtS", "Polling", "1.0 .5 0");
+	poti_DefineEntityValue("Dr", "CtS", "DriverRun", ".1 .1 1.0");
 	poti_DefineEntityValue("Sl", "CtS", "Sleeping", ".9 .1 .0");
 	poti_DefineEntityValue("UT", "CtS", "UserTesting", ".2 .1 .6");
 	poti_DefineEntityValue("UW", "CtS", "UserWaiting", ".4 .1 .3");
 	poti_DefineEntityValue("SdS", "CtS", "SendSubmitted", "1.0 .1 1.0");
-	poti_DefineEntityValue("RvS", "CtS", "RecieveSubmitted", "0.1 1.0 1.0");
+	poti_DefineEntityValue("RvS", "CtS", "ReceiveSubmitted", "0.1 1.0 1.0");
 	poti_DefineEntityValue("SdC", "CtS", "SendCompleted", "1.0 .5 1.0");
 	poti_DefineEntityValue("RvC", "CtS", "ReceiveCompleted", "0.5 1.0 1.0");
 	poti_DefineEntityValue("TD", "CtS", "Testing Detached", ".0 .0 .6");
 	poti_DefineEntityValue("MT", "CtS", "MPI Test", ".0 .0 .8");
 	poti_DefineEntityValue("Bu", "CtS", "Building task", ".5 .18 .0");
-	poti_DefineEntityValue("Su", "CtS", "Submiting task", ".3 .09 .0");
+	poti_DefineEntityValue("Su", "CtS", "Submitting task", ".3 .09 .0");
 	poti_DefineEntityValue("Th", "CtS", "Throttling task submission", ".8 .6 .6");
 	poti_DefineEntityValue("C", "CtS", "Callback", ".0 .3 .8");
 
@@ -253,8 +359,9 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 	poti_DefineEventType("user_thread_event", "UT", "thread event type");
 	poti_DefineStateType("US", "UT", "User Thread State");
 	poti_DefineEntityValue("Bu", "US", "Building task", ".5 .18 .0");
-	poti_DefineEntityValue("Su", "US", "Submiting task", ".3 .09 .0");
+	poti_DefineEntityValue("Su", "US", "Submitting task", ".3 .09 .0");
 	poti_DefineEntityValue("C", "US", "Callback", ".0 .3 .8");
+	poti_DefineEntityValue("Sc", "US", "Scheduling", ".7 .36 .0");
 	poti_DefineEntityValue("Th", "US", "Throttling task submission", ".8 .6 .6");
 	poti_DefineEntityValue("MD", "US", "Decoding task for MPI", ".5 .18 .2");
 	poti_DefineEntityValue("MPr", "US", "Preparing task for MPI", ".4 .14 .2");
@@ -268,7 +375,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 		char inctx[10];
 		snprintf(inctx, sizeof(inctx), "InCtx%u", i);
 		char *ctx = inctx+2;
-		poti_DefineStateType(ctx, "T", inctx);
+		poti_DefineStateType(ctx, "W", inctx);
 		poti_DefineEntityValue("I", ctx, "Idle", ".9 .1 .0");
 		poti_DefineEntityValue("In", ctx, "Initializing", "0.0 .7 1.0");
 		poti_DefineEntityValue("D", ctx, "Deinitializing", "0.0 .1 .7");
@@ -310,6 +417,8 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 1       MPICt   P       \"MPI Communication Thread\"              \n\
 1       Sc       P       \"Scheduler State\"                        \n\
 2       prog_event   P       \"program event type\"				\n\
+2       pu   P       \"Task Push\"                             \n\
+2       po   P       \"Task Pop\"                              \n\
 2       register     P       \"data registration\"				\n\
 2       unregister     P       \"data unregistration\"				\n\
 2       user_event   P       \"user event type\"				\n\
@@ -320,16 +429,34 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 3       S       T       \"Thread State\"                        \n\
 3       CtS     MPICt    \"Communication Thread State\"          \n");
 	for (i=1; i<STARPU_NMAX_SCHED_CTXS; i++)
-		fprintf(file, "3       Ctx%u      T     \"InCtx%u\"         		\n", i, i);
+		fprintf(file, "3       Ctx%u      W     \"InCtx%u\"         		\n", i, i);
 	fprintf(file, "\
-2       invalidate Mm \"data invalidation\"                            \n\
+2       SI       Mm \"data state invalid\"                            \n\
+2       SS       Mm \"data state shared\"                            \n\
+2       SO       Mm \"data state owner\"                            \n\
+2       WU       Mm \"data wont use\"                            \n\
+2       Al       Mm    \"Allocating Start\"    \n\
+2       rc       Mm    \"Request Created\"    \n\
+2       AlE      Mm    \"Allocating End\"    \n\
+2       Alr      Mm    \"Allocating Async Start\"    \n\
+2       AlrE     Mm    \"Allocating Async End\"    \n\
+2       Fe       Mm    \"Free Start\"    \n\
+2       FeE      Mm    \"Free End\"    \n\
+2       Wb       Mm    \"WritingBack Start\"    \n\
+2       WbE      Mm    \"WritingBack End\"    \n\
+2       DCo      Mm    \"DriverCopy Start\"    \n\
+2       DCoE     Mm    \"DriverCopy End\"    \n\
+2       DCoA     Mm    \"DriverCopyAsync Start\"    \n\
+2       DCoAE    Mm    \"DriverCopyAsync End\"    \n\
 3       MS       Mm       \"Memory Node State\"                        \n\
 4       nsubmitted    Sc       \"Number of Submitted Uncompleted Tasks\"                        \n\
 4       nready    Sc       \"Number of Ready Tasks\"                        \n\
 4       gft    Sc       \"Total GFlops\"                        \n\
 4       use     Mm       \"Used (MB)\"                        \n\
-4       bwi     Mm       \"Bandwidth In (MB/s)\"                        \n\
-4       bwo     Mm       \"Bandwidth Out (MB/s)\"                        \n\
+4       bwi_mm     Mm       \"Bandwidth In (MB/s)\"                        \n\
+4       bwo_mm     Mm       \"Bandwidth Out (MB/s)\"                        \n\
+4       bwi_mpi     MPICt       \"Bandwidth In (MB/s)\"                        \n\
+4       bwo_mpi     MPICt       \"Bandwidth Out (MB/s)\"                        \n\
 4       gf      W       \"GFlops\"                        \n\
 6       I       S       Idle         \".9 .1 .0\"		\n\
 6       In       S      Initializing       \"0.0 .7 1.0\"            \n\
@@ -345,7 +472,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 6       U       S       Unpartitioning      \".0 .0 1.0\"		\n\
 6       H       S       Hypervisor      \".5 .18 .0\"		\n\
 6       Bu      S       \"Building task\"   \".5 .18 .0\"		\n\
-6       Su      S       \"Submittings task\" \".3 .09 .0\"		\n\
+6       Su      S       \"Submitting task\" \".3 .09 .0\"		\n\
 6       Th      S       \"Throttling task submission\" \".8 .6 .6\"		\n\
 6       MD      S       \"Decoding task for MPI\" \".5 .18 .2\"		\n\
 6       MPr     S       \"Preparing task for MPI\" \".4 .14 .2\"		\n\
@@ -369,8 +496,9 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 6       Th      WS       \"Throttling task submission\" \".8 .6 .6\"		\n\
 3       US       UT       \"User Thread State\"                        \n\
 6       Bu      US      \"Building task\"   \".5 .18 .0\"		\n\
-6       Su      US      \"Submittings task\" \".3 .09 .0\"		\n\
+6       Su      US      \"Submitting task\" \".3 .09 .0\"		\n\
 6       C       US      \"Callback\" \".0 .3 .8\"		\n\
+6       Sc      US      \"Scheduling\"         \".7 .36 .0\"		\n\
 6       Th      US      \"Throttling task submission\" \".8 .6 .6\"		\n\
 6       MD      US      \"Decoding task for MPI\" \".5 .18 .2\"		\n\
 6       MPr     US      \"Preparing task for MPI\" \".4 .14 .2\"		\n\
@@ -381,17 +509,19 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 ");
 	fprintf(file, "\
 6       P       CtS       Processing         \"0 0 0\"		\n\
+6       Pl       CtS      Polling	   \"1.0 .5 0\"		\n\
+6       Dr       CtS      DriverRun	   \".1 .1 1.0\"	\n\
 6       Sl       CtS      Sleeping         \".9 .1 .0\"		\n\
 6       UT       CtS      UserTesting        \".2 .1 .6\"	\n\
 6       UW       CtS      UserWaiting        \".4 .1 .3\"	\n\
 6       SdS       CtS      SendSubmitted     \"1.0 .1 1.0\"	\n\
-6       RvS       CtS      RecieveSubmitted  \"0.1 1.0 1.0\"	\n\
+6       RvS       CtS      ReceiveSubmitted  \"0.1 1.0 1.0\"	\n\
 6       SdC       CtS      SendCompleted     \"1.0 .5 1.0\"	\n\
 6       RvC       CtS      ReceiveCompleted  \"0.5 1.0 1.0\"	\n\
 6       TD       CtS      \"Testing Detached\"  \".0 .0 .6\"	\n\
 6       MT       CtS      \"MPI Test\"  \".0 .0 .8\"	\n\
 6       Bu      CtS      \"Building task\"   \".5 .18 .0\"		\n\
-6       Su      CtS      \"Submittings task\" \".3 .09 .0\"		\n\
+6       Su      CtS      \"Submitting task\" \".3 .09 .0\"		\n\
 6       Th      CtS      \"Throttling task submission\" \".8 .6 .6\"		\n\
 6       C       CtS      \"Callback\" \".0 .3 .8\"		\n\
 ");
@@ -408,8 +538,9 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 6       Sc       Ctx%u      Scheduling         \".7 .36 .0\"		\n\
 6       Sl       Ctx%u      Sleeping         \".9 .1 .0\"		\n\
 6       P       Ctx%u       Progressing         \".1 .3 .1\"		\n\
-6       U       Ctx%u       Unpartitioning         \".0 .0 1.0\"		\n",
-		i, i, i, i, i, i, i, i, i, i, i, i);
+6       U       Ctx%u       Unpartitioning         \".0 .0 1.0\"	\n\
+6       H       Ctx%u       Hypervisor         \".5 .18 .0\"		\n",
+		i, i, i, i, i, i, i, i, i, i, i, i, i);
 	fprintf(file, "\
 6       A       MS      Allocating         \".4 .1 .0\"		\n\
 6       Ar       MS      AllocatingReuse       \".1 .1 .8\"		\n\
@@ -426,7 +557,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED)
 5       IF      P	Mm	Mm      \"Intra-node data IdleFetch\"\n\
 5       WSL     P	W	W       \"Work steal\"\n");
 
-	fprintf(file, "7      0.0 MPIroot      MPIP      0       root\n");
+	fprintf(file, "7	0.0 MPIroot      MPIP      0       root\n");
 #endif
 }
 
